 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Panel Element                           *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/

	/**
	 * Class: thyPanel
	 *
	 *	Base class for all other ThyAPI GUI Components
	 *
	 *	This class is the parent of all other ThyAPI GUI components. It
	 *	performs some internal initializations. It garantees, for example, that
	 *	all elements have 2 CSS classes. One with the name of the class, for
	 *	commom object definitions and other for the specific object name,
	 *	that is passed as first argument of all classes constructor.
	 *
	 *	CSS Behavior:
	 *
	 *	thyPanel has 3 internal variables that controls which CSS classes a widget
	 *	will receive.
	 *
	 *	cssBaseClasses - A thyCollection of strings that are the base name of widget
	 *	                 CSS classes. This is used to allow an hierarchy of CSS classes.
	 *	                 When a widget B is defined as a derivative of A, and it calls
	 *	                 addCSSClass('B') method, this method inserts the 'B' string into
	 *	                 the collection and guarantees that all CSS change operations
	 *	                 includes CSS classes with base names 'A' and 'B', so all common
	 *	                 CSS properties of 'A' are inheritted by 'B' automagically!
	 *
	 *	cssClasses - A simple JS object that holds the specific types of classes the widget
	 *	             can have. This is used to change visual states on widgets. For instance,
	 *	             let's imagine the previous situation of 'B' derivate from 'A'. Let's
	 *	             suppose 'B' has a 'selected' state. In this case, one could define an
	 *	             entry in cssClasses like cssClasses[selected] = ['default', 'selected'].
	 *	             In this case, one could change the visual appearance of 'B' calling
	 *	             setCSSClass('selected'). In this case, this would set widget CSS classes
	 *	             to 'A B A_selected B_selected'. That means that it takes all cssBaseClasses
	 *	             and append them with each one of the strings in the cssClasses['selected']
	 *	             array, with exception to 'default', which is reserved and means 'just the
	 *	             base class'.
	 *
	 *	cssExtraClasses - Another simple JS object that is meant for temporary CSS classes.
	 *	                  cssClasses is meant to be used as a fixed place for types of classes
	 *	                  that corresponds to visual state changes.
	 *	                  cssExtraClasses allows one to define extra strings that will be appended
	 *	                  to cssBaseClasses, but that are not fixed to a state. This is used mainly
	 *	                  by onmouseover, onmouseout events, but can be used elsewhere and independently
	 *	                  of cssClasses. To use it, define a holder and in that holder define an object
	 *	                  with types index and array of strings as contents.
	 *
	 *	                  Example:
	 *	                  cssExtraClasses['egHolder']['selected'] = ['selected_diferent'];
	 *
	 *	                  When 'egHolder' extra classes are not needed anymore, just remove it from
	 *	                  cssExtraClasses and calls setCSSClass('selected');
	 *
	 * Author:
	 *
	 *	Raphael Derosso Pereira <raphaelpereira@users.sourceforge.net>
	 *
	 * Events:
	 *
	 *	onparentchange - Invoked when any other <thyPanel> or derivate class is
	 *	inserted as a child.
	 *	onbeforeparentchange - Invoked before changing the parent of a child widget
	 *
	 */
	function thyPanel(name,html)
	{
		this.thyBase = thyBase;
		this.thyBase();
		this.DynLayer = DynLayer;
		this.DynLayer(html);
		this.setOverflow('');

		// Attributes \\
		this.x = null;
		this.y = null;
		this.width = null;
		this.height = null;
		this.isThyPanel = true;
		this._cursor = null;
		this._position = null;
		this.name = name || this._className+(this._nameCount++);
		this.readOnly = false;
		this.disabled = false;
		this.contentRelative = true;
		this.noBreak = false;
		this.caption = null;

		// CSS Attributes \\
		this.cssBaseClasses = new thyCollection();
		
		this.cssClasses = {'default' : 'default'};
		this.cssExtraClasses = {};
		this.cssActualSelection;
		
		// Initialize CSS classes \\
		this.addCSSClass('thyPanel');

		var self = this;
		this.onCreate(function(){
			self.setCSSClass(self.cssActualSelection);
		});
	}
	
	p = dynapi.setPrototype('thyPanel',['DynLayer','thyBase']);
	p._nameCount = 0;

	/*************************************************************************\
	 *                         Group: Data Methods                           *
	\*************************************************************************/
	
	/**
	 * Method: populate
	 *
	 *	Method that will be called by DataSource when data comes from server
	 *
	 *	This method MUST be overwritten by derivate classes to achieve its proper
	 *	populate status. The method implemented here, checks if the data is a string
	 *	or an object. If it is string, inserts it as inner HTML. If it is an object,
	 *	it checks if it has children with the name of each components
	 *
	 * Parameter:
	 * 
	 *	data - The data to be populated
	 *
	 */
	p.populate = function(data)
	{
		if (typeof(data) == 'string')
		{
			this.setHTML(data);
			return;
		}
			
		if (typeof(data) != 'object') return;

		var i, ch = this.children;
		for (i=0; i<ch.length; i++)
		{
			if (ch[i].name && ch[i].populate && data[ch[i].name]) ch[i].populate(data[i]);
		}
	}

	/**
	 * Method: sweepOut
	 *
	 *	Method that will be called by DataSource before sending data to
	 *	server
	 *
	 * Returns:
	 *
	 *	Array with the data of the field
	 *
	 */
	p.sweepOut = function()
	{
		var i, response={},ch=this.children;

		if (!ch.length) return this.getHTML();
		
		for (i=0; i<ch.length; i++)
		{
			if (ch[i].name && ch[i].sweepOut) response[ch[i].name] = ch[i].sweepOut();
		}
		
		return response;
	}
	
	/**
	 * Method: cleanUp
	 *
	 *	Removes all data from panel and from its children
	 *
	 */
	p.cleanUp = function ()
	{
		if (this.children)
		{
			var i;
			for (i=0; i<this.children.length; i++)
			{
				if (this.children[i].cleanUp) this.children[i].cleanUp();
			}
		}
		this._cleanUp();
	}

	/**
	 * Method: reset
	 *
	 *	Removes all transient data from panel
	 *
	 */
	p.reset = function ()
	{
		if (this.children)
		{
			var i;
			for (i=0; i<this.children.length; i++)
			{
				if (this.children[i].reset) this.children[i].reset();
			}
		}
		this._reset();
	}

	/*************************************************************************\
	 *                         Group: Misc Methods                           *
	\*************************************************************************/
	
	/**
	 * Method: setName
	 *
	 *	Changes the name of the widget and refreshes the CSS Classes
	 *
	 * Parameter:
	 *
	 *	name - The new widget name
	 *
	 */
	p.setName = function (name)
	{
		if (name == null || this.name == name) return;
		
		this.name = name;
		this.setCSSClass(this.cssActualSelection);
	}
	
	p._uberInvokeEvent = p.invokeEvent;
	/**
	 * Method: invokeEvent
	 *
	 *	Overwritten invokeEvent method to detect disabled state
	 *
	 */
	p.invokeEvent = function(type,e,args)
	{
		if (this.disabled) return;
		this._uberInvokeEvent(type,e,args);
	}
	
	/*************************************************************************\
	 *                        Group: Visual Methods                          *
	\*************************************************************************/
	
	/**
	 * Method: setDOMParent
	 *
	 *	Attach this widget to a DOM element. This means that this panel
	 *	will be inserted in the DOM structure below the specified DOM
	 *	element
	 *
	 * Parameters:
	 *
	 *	element - The DOM element to be attached
	 *
	 * Prerequisites:
	 *
	 *  dynapi.api.ext.DynLayerInline
	 */
	p.setDOMParent = function (inlineID)
	{
		var inlineLayer = DynLayer.getInline(inlineID); //dynapi.document.addChild(new DynLayer(),null,inl);
		inlineLayer.addChild(this);
	}
	
	p._DynLayerAddChild = p.addChild;
	/**
	 * Method: addChild
	 *
	 *	Add another object inside this one.
	 *
	 *	This method is overwritten here so it permits the insertion
	 *	of a element in a specified position
	 *
	 * Parameters:
	 *
	 *	child - The element itself
	 *	position - The position in which this element should be inserted
	 *
	 * Invokes:
	 *
	 *	- onbeforeparentchange
	 *	- onparentchange
	 *
	 */
	p.addChild = function (child, position)
	{
		child.invokeEvent('beforeparentchange');
		if (position != null && typeof(position) == 'number' && position < this.children.length)
		{
			this._DynLayerAddChild(child);
			
			var start = this.children.slice(0, position);
			var end   = this.children.slice(position, -1);
			this.children = start.concat(child,end);
			
			// Elements not created. Just reposition it on array \\
			if (this._created)
			{
				var elm = this._getDOMChildrenHolder();

				var oldElm = elm.childNodes[position];
				elm.insertBefore(child.elm, oldElm);
			}
		}
		else
		{
			this._DynLayerAddChild(child);
		}

		child.setDisabled(this.disabled);
		//child.setReadOnly(this.readOnly);
		child.invokeEvent('parentchange');
	}

	p._DynLayerRemoveChild = p.removeChild;
	/**
	 * Method: removeChild
	 *
	 *	Removes a child from Panel
	 *
	 * Parameter:
	 *
	 *	child - The child widget that will be removed or its index
	 *
	 */
	p.removeChild = function (child)
	{
		if (typeof(child) == 'number') child = this.children[child];

		this._DynLayerRemoveChild(child);
	}
	
	/**
	 * Method: setReadOnly
	 *
	 *	Sets this element's read-only attribute
	 *
	 *	Derivate classes should implement <_setReadOnly> specific
	 *	method
	 *
	 * Parameter:
	 *
	 *	bool - True -> read-only, false -> read/write
	 *
	 */
	p.setReadOnly = function(bool)
	{
		this.readOnly = (bool) ? true:false;
		if (this.children)
		{
			var i;
			for (i=0; i<this.children.length; i++)
			{
				if (this.children[i].setReadOnly) this.children[i].setReadOnly(this.readOnly);
			}
		}
		this._setReadOnly();
	}

	/**
	 * Method: setDisabled
	 *
	 *	Sets this element's disabled attribute
	 *
	 *	Derivate classes should implement _setDisabled specific
	 *	method
	 *
	 * Parameter:
	 *
	 *	bool - true -> disabled, false -> enabled
	 *
	 */
	p.setDisabled = function(bool)
	{
		this.disabled = (bool) ? true:false;

		// FIXME: Could not override this class in specific file!!!
		if (dynapi.ua.ie)
		{
			if (!this._created) return;
			this.elm.disabled = this.disabled;
			return;
		}

		if (this.children)
		{
			var i;
			for (i=0; i<this.children.length; i++)
			{
				if (this.children[i].setDisabled) this.children[i].setDisabled(this.disabled);
			}
		}
		if (this._created) this.elm.disabled = this.disabled;
		if (this._setDisabled) this._setDisabled();
	}

	/**
	 * Method: setDisplay
	 *
	 * Sets this element's display property
	 *
	 * Parameters:
	 *
	 *	display - (string) The display property value
	 *
	 */
	p.setDisplay = function (display)
	{
		if (typeof(display) == 'boolean') display ? display='' : display='none';
		if (!display && typeof(display) != 'string') display = '';
		this.display = display;
		if (this.css)
		{
			this.css.display = display;
			this._invokeOnDisplay();
		}
	}

	/**
	 * Method: getDisplay
	 *
	 *	Returns this element's display property
	 *
	 *	FIXME: Verify if parent is set to display none!
	 */
	p.getDisplay = function ()
	{
		return this.display;
	}

	/**
	 * Method: setRelativeness
	 *
	 *	Sets this element internal ralative box creation
	 *
	 * Parameter:
	 *
	 *	bool - True -> no internal relative box, False -> internal relative box
	 */
	p.setRelativeness = function (b)
	{
		this.contentRelative = (b) ? true : false;
	}

	/**
	 * Method: setNoBreak
	 *
	 * If set to true, a <nobr> tag will be inserted around contents
	 *
	 * Parameter:
	 *
	 *	bool - True / false
	 *
	 */
	p.setNoBreak = function(bool)
	{
		this.noBreak = (bool) ? true : false;
	}

	/**
	 * Method: setFloat
	 *
	 *	Sets the CSS Float property
	 *
	 * Parameter:
	 *
	 *	f - (string) The float property
	 *
	 */
	p.setFloat = function (f)
	{
		this._float = f;
		this._cssFloat = ' float: '+f+';';
		
		if (this._crated) this.css.cssFloat = f;
	}

	/**
	 * Method: getOuterHTML
	 *
	 *	Overwritten DynLayer Methods that garantees CSS priority when creating elements
	 *
	 */
	p.getOuterHTML = function ()
	{
		var html = [
			'<div '+this._cssClass+' id="'+this.id+'" '+(this.disabled ? 'disabled' : '')+' '+(this.readOnly?'readonly':'')+' style="',
			((this._position!=null) ? ' position: '+this._position+';' : ''),
			((this.x!=null)? ' left: '+((typeof(this.x)=='string')?this.x+';':this.x+'px;') : ''),
			((this.y!=null)? ' top: '+((typeof(this.y)=='string')?this.y+';':this.y+'px;') : ''),		
			((this.w)? ' width:'+((typeof(this.w)=='string')?this.w+';':(this.w+fixBw)+'px;'):''),
			((this.h)? ' height:'+((typeof(this.h)=='string')?this.h+';':(this.h+fixBh)+'px;'):''),
			((this.z)? ' z-index: '+this.z+';':''),
			((this._cursor!=null)? ' cursor:'+this._cursor+';':''),
			((this.visible==false)? ' visibility:hidden;': ''),
			((this.display) ? ' display: '+this.display+';' : ''),
			this._cssBorder,
			((this._overflow)? this._cssOverflow : ''),
			((this.x != null || this.y != null) ? this._cssPosition+';' : ''),
			(this._float ? this._cssFloat : ''),
			//(this.contentRelative ? '"><div class="'+this.name+'_relative'+'" style="position: relative; width: 100%; height: 100%">' : '">'),
			(this.contentRelative ? '"><div class="'+this.name+'_relative'+'" style="">' : '">'),
			(this.noBreak ? '<nobr>' : ''),
			this.getInnerHTML(),
			(this.noBreak ? '</nobr>' : ''),
			(this.contentRelative ? '</div></div>' : '</div>')
		].join('');
		//alert(html);
		return html;
	}

	/**
	 * Method: getAbsoluteX
	 *
	 *	Returns the actual X position of this element, relative
	 *	to the whole page
	 *
	 */
	p.getAbsoluteX = function ()
	{
		if (this._created)
		{
			var elm = this.elm;
			var x = elm.offsetLeft;

			while (elm.offsetParent)
			{
				elm = elm.offsetParent;
				x += elm.offsetLeft;
			}

			return x;
		}

		return null;
	}

	/**
	 * Method: getAbsoluteY
	 *
	 *	Returns the actual Y position of this element, relative
	 *	to the whole page
	 *
	 */
	p.getAbsoluteY = function ()
	{
		if (this._created)
		{
			var elm = this.elm;
			var y = elm.offsetTop;

			while (elm.offsetParent)
			{
				elm = elm.offsetParent;
				y += elm.offsetTop;
			}

			return y;
		}

		return null;
	}
	
	/**
	 * Method: getX
	 *
	 *	Returns the actual X position of this element, relative to
	 *	its parent
	 *
	 */
	p.getX = function ()
	{
		var x = this.getAbsoluteX();
		if (this._created)
		{
			var elm = this.elm.offsetParent;
			
			while(elm.offsetParent)
			{
				x -= elm.offsetLeft;
				elm = elm.offsetParent;
			}

			return x;
		}
		return x;
	}

	/**
	 * Method: setX
	 *
	 *	Sets X position and calls locationchange event
	 *
	 * Parameter:
	 *
	 *	x - The X position. That can be an integer (number of pixels)
	 *	    or a string that can be used to set %
	 *
	 */
	p.setX = function (x)
	{
		this.x = typeof(x) == 'string' ? x : x+'px';
		if (!this._created) return;
		this.css.left = this.x;
		if(this._hasLocationEvents) this.invokeEvent('locationchange', new DynEvent('locationchange',this));
	}

	/**
	 * Method: getY
	 *
	 *	Returns the actual Y position of this element, relative to
	 *	its parent
	 *
	 */
	p.getY = function ()
	{
		var y = this.getAbsoluteY();
		if (this._created)
		{
			var elm = this.elm.offsetParent;
			
			while(elm.offsetParent)
			{
				y -= elm.offsetLeft;
				elm = elm.offsetParent;
			}

			return y;
		}
		return y;
	}

	/**
	 * Method: setY
	 *
	 *	Sets Y position and calls locationchange event
	 *
	 * Parameter:
	 *
	 *	y - The Y position. That can be an integer (number of pixels)
	 *	    or a string that can be used to set %
	 *
	 */
	p.setY = function (y)
	{
		this.y = typeof(y) == 'string' ? y : y+'px';
		if (!this._created) return;
		this.css.top = this.y;
		if(this._hasLocationEvents) this.invokeEvent('locationchange', new DynEvent('locationchange',this));
	}

	/**
	 * Method: getHeight
	 *
	 *	Returns the actual height of the panel
	 *
	 */
	p.getHeight = function ()
	{
		if (this._created)
		{
			return this.elm.offsetHeight;
		}

		return null;
	}
	
	/**
	 * Method: getWidth
	 *
	 *	Returns the actual width of the panel
	 *
	 */
	p.getWidth = function ()
	{
		if (this._created)
		{
			return this.elm.offsetWidth;
		}

		return null;
	}

	/**
	 * Method: setHeight
	 *
	 *	Sets element height.
	 *
	 *	*DO NOT USE THIS METHOD UNLESS YOU KNOW WHAT YOU ARE DOING!*
	 *	It will overwrite CSS properties, which is not preferred in thyAPI
	 *
	 */
	p.setHeight = function (h)
	{
		if (this._created)
		{
			if (this.css) var css = this.css;
			else var css = this;
			
			if (typeof(h) == 'string') css.height = h;
			else css.height = h+'px';

			return;
		}

		if (typeof(h) == 'string') this.h = h;
		else this.h = h+'px';
	}
	
	/**
	 * Method: setWidth
	 *
	 *	Sets element width.
	 *
	 *	*DO NOT USE THIS METHOD UNLESS YOU KNOW WHAT YOU ARE DOING!*
	 *	It will overwrite CSS properties, which is not preferred in thyAPI
	 *
	 */
	p.setWidth = function (w)
	{
		if (this._created)
		{
			if (this.css) var css = this.css;
			else var css = this;
			
			if (typeof(w) == 'string') css.width = w;
			else css.width = w+'px';

			return;
		}

		if (typeof(w) == 'string') this.w = w;
		else this.w = w+'px';
	}
	
	/**
	 * Method: setMinWidth
	 *
	 *	Sets element min-width CSS property.
	 *
	 *	*DO NOT USE THIS METHOD UNLESS YOU KNOW WHAT YOU ARE DOING!*
	 *	It will overwrite CSS properties, which is not preferred in thyAPI
	 *
	 */
	p.setMinWidth = function (w)
	{
		if (this._created)
		{
			if (this.css) var css = this.css;
			else var css = this;
			
			if (typeof(w) == 'string') css.minWidth = w;
			else css.minWidth = w+'px';

			return;
		}

		// FIXME: This is wrong. There should be an minW property
		if (typeof(w) == 'string') this.w = w;
		else this.w = w+'px';
	}
	
	/**
	 * Method: setMinHeight
	 *
	 *	Sets element min-height CSS property.
	 *
	 *	*DO NOT USE THIS METHOD UNLESS YOU KNOW WHAT YOU ARE DOING!*
	 *	It will overwrite CSS properties, which is not preferred in thyAPI
	 *
	 */
	p.setMinHeight = function (h)
	{
		if (this._created)
		{
			if (this.css) var css = this.css;
			else var css = this;
			
			if (typeof(h) == 'string') css.minHeight = h;
			else css.minHeight = h+'px';

			return;
		}

		// FIXME: This is wrong. There should be an minW property
		if (typeof(h) == 'string') this.h = h;
		else this.h = h+'px';
	}
	
	/*************************************************************************\
	 *                          Group: CSS Methods                           *
	\*************************************************************************/
	
	/**
	 * Method: getCSSClasses
	 *
	 *	Returns the CSS Classes based on cssBaseClasses and append aditional parameters,
	 *	if available
	 *
	 * Parameter:
	 *
	 *	type - The index of cssClasses which hold the classes types to be returnes
	 *
	 */
	p.getCSSClasses = function (type)
	{
		var i;
		var classes = new Array();
		var types = new Array();
		
		if (!type) type = 'default';

		if (!this.cssClasses[type])
		{
			if (type != 'default') return;
			else types = [];
		}
		else types = this.cssClasses[type];

		if (typeof(types) == 'string') types = [types];

		classes = this._appendCSSClasses(classes, types, this.cssBaseClasses.getIndexedElements());

		// Check for extra classes
		if (this.cssExtraClasses)
		{
			var extra = this.cssExtraClasses;
			var ref;
			for (i in extra)
			{
				if (extra[i][type])
				{
					if (extra[i]['refClasses']) ref = extra[i]['refClasses'];
					else ref = this.cssBaseClasses.getIndexedElements();
					classes = this._appendCSSClasses(classes, extra[i][type], ref);
				}
			}
		}

		return classes.reverse().join(' ');
	}

	/**
	 * Method: addCSSClass
	 *
	 *	Append a base class to the base classes list and refresh
	 *
	 * Parameter:
	 *
	 *	cssClass - The name of the class that will be appended
	 *
	 */
	p.addCSSClass = function (cssClass)
	{
		this._addCSSClass(cssClass);
		this.setCSSClass(this.cssActualSelection);
	}

	/**
	 * Method: removeCSSClass
	 *
	 *	Removes the CSS Class from the list of Classes
	 *
	 * Parameter:
	 *
	 *	cssClass - The CSS Class to be removed
	 *
	 */
	p.removeCSSClass = function (cssClass)
	{
		this._removeCSSClass(cssClass);
		this.setCSSClass(this.cssActualSelection);
	}
	
	/**
	 * Method: setCSSClass
	 *
	 *	Sets the CSS Class property of this widget based on its base classes,
	 *	cssClasses attributes and extra classes
	 *
	 * Parameter:
	 *
	 *	type - setCSSClass
	 *
	 */
	p.setCSSClass = function(type)
	{
		if (!type) type = 'default';

		this.cssActualSelection = type;
		
		if (!this._created) return;

		this.setClass(this.getCSSClasses(type));
	}

	/**
	 * Method: setClass
	 *
	 *	Overloaded method that sets the CSS class of the element
	 *
	 *
	 * Parameter:
	 *
	 *	cssClasses - String with CSS classes
	 *
	 *
	p.setClass = function (cssClasses)
	{
		var cssClasses = cssClasses.split(' ');
		var actualClasses = this.elm.className;
		var i;
		for (i=0; i<cssClasses.length; i++)
		{
		}
	}*/
	
	/*************************************************************************\
	 *                         Group: Private Methods                        *
	\*************************************************************************/

	/**
	 * Method: _getDOMChildrenHolder
	 *
	 *	Returns the DOM object that holds children DOM objects
	 *
	 */
	p._getDOMChildrenHolder = function ()
	{
		if (!this._created) return null;

		var elm;
		
		if (this.contentRelative)
		{
			elm = this.elm.childNodes[0];
		}
		else
		{
			elm = this.elm;
		}

		return elm;
	}
	
	/**
	 * Method: _cleanUp
	 *
	 *	Removes all data from panel
	 *
	 */
	p._cleanUp = function ()
	{
		if (this.children) return;
		
		this.setHTML('');
	}

	/**
	 * Method: _reset
	 *
	 *	Removes all transient data from panel
	 *
	 */
	p._reset = function ()
	{
		if (this.children.length) return;

		this.setHTML('');
	}
	
	/**
	 * Method: _setReadOnly
	 *
	 *	Specific <thyPanel> function to set Panel to read-only state.
	 *
	 *	This should be reimplemented in derivate classes.
	 *
	 */
	p._setReadOnly = function()
	{
		if (!this.elm) return;
		//this.elm.setAttribute('readonly', '');
		this.elm.readOnly = this.readOnly;
	}
	
	/**
	 * Method: _setDisabled
	 *
	 *	Specific <thyPanel> function to disable Panel.
	 *
	 *	This should be reimplemented in derivate class to reflect
	 *	their real need.
	 *
	 *
	p._setDisabled = function()
	{
		if (this.disabled)
			this.addCSSClass('thyPanel_disabled');
		else
			this.removeCSSClass('thyPanel_disabled');
	}*/
	
	/**
	 * Method: _invokeOnDisplay
	 *
	 *	Invokes ondisplay event handler on myself and all my children
	 *
	 */
	p._invokeOnDisplay = function ()
	{
		this.invokeEvent('display', new DynEvent('display', this));
		
		var i=0;
		for (i=0; i<this.children.length; i++)
		{
			if (typeof(this.children[i]._invokeOnDisplay) == 'function')
			{
				this.children[i]._invokeOnDisplay();
			}
		}
	}
	
	/**
	 * Method: _addCSSClass
	 *
	 *	Adds the specified CSS Class to the list of classes
	 *
	 * Parameters:
	 *
	 *	cssClass - The class to be added
	 *
	 */
	p._addCSSClass = function (cssClass)
	{
		this.cssBaseClasses.add(cssClass, cssClass);
	}
	
	/**
	 * Method: _removeCSSClass
	 *
	 *	Removes the CSS Class from the list of Classes
	 *
	 * Parameter:
	 *
	 *	cssClass - The CSS Class to be removed
	 *
	 */
	p._removeCSSClass = function (cssClass)
	{
		this.cssBaseClasses.remove(cssClass);
	}

	/**
	 * Method: _appendCSSClasses
	 *
	 */
	p._appendCSSClasses = function (classes, types, base)
	{
		var i,j,cssClass;
		
		if (typeof(types) == 'string') types = [types];

		for (i=0; i<base.length; i++)
		{
			for (j=0; j<types.length; j++)
			{
				cssClass = types[j] != 'default' ? base[i]+'_'+types[j] : base[i];
				classes.push(cssClass);
				switch (dynapi.ua.ua)
				{
					case 'ie': classes.push(cssClass+'_ie'); break;
					case 'opera': classes.push(cssClass+'_opera'); break;
					case 'safari': classes.push(cssClass+'_safari'); break;
					case 'ns': classes.push(cssClass+'_ns'); break;
				}
			}
		}

		for (j=0; j<types.length; j++)
		{
			cssClass = types[j] != 'default' ? this.name+'_'+types[j] : this.name;
			classes.push(types[j] != 'default' ? this.name+'_'+types[j] : this.name);
			switch (dynapi.ua.ua)
			{
				case 'ie': classes.push(cssClass+'_ie'); break;
				case 'opera': classes.push(cssClass+'_opera'); break;
				case 'safari': classes.push(cssClass+'_safari'); break;
				case 'ns': classes.push(cssClass+'_ns'); break;
			}
		}
		
		return classes;
	}

	/**
	 * Method: _concatCSSClasses
	 *
	 */
	p._concatCSSClasses = function (classes, concat)
	{
		if (typeof(classes) == 'string') classes = [classes];
		var i;
		var newClasses = [];
		for (i=0; i<classes.length; i++)
		{
			newClasses.push(classes[i] != 'default' ? classes[i]+'_'+concat : concat);
		}
		return newClasses;
	}

	/*************************************************************************\
	 *                              Events                                   *
	\*************************************************************************/

	/**
	 * Method: onmouseover
	 *
	 *	This garantees that all panels and derivate classes have the
	 *	class set to <className>_onmouseover and <name>_onmouseover
	 *
	 */
	p.onmouseover = function (e)
	{
		var actual;
		if (!this.cssClasses || (actual = this.cssClasses[this.cssActualSelection]) == null) return;
		var extra = {};
		extra[this.cssActualSelection] = this._concatCSSClasses(actual,'onmouseover');
		this.cssExtraClasses['onmouseover'] = extra ;
		this.setCSSClass(this.cssActualSelection);
	}

	/**
	 * Method: onmouseout
	 *
	 *	This garantees that all panels and derivate classes have the
	 *	class set to <className> and <name>
	 *
	 */
	p.onmouseout = function (e)
	{
		var actual;
		if (!this.cssClasses || (actual = this.cssClasses[this.cssActualSelection]) == null) return;
		delete this.cssExtraClasses['onmouseover'];
		this.setCSSClass(this.cssActualSelection);
	}

	/**
	 * Method: onmousedown
	 *
	 *	This garantees that all panels and derivate classes have the
	 *	class set to <className>_onmousedown and <name>_onmousedown
	 *
	 */
	p.onmousedown = function (e)
	{
		var actual;
		if (!this.cssClasses || (actual = this.cssClasses[this.cssActualSelection]) == null) return;
		var extra = {};
		extra[this.cssActualSelection] = this._concatCSSClasses(actual,'onmousedown');
		this.cssExtraClasses['onmousedown'] = extra;
		this.setCSSClass(this.cssActualSelection);
	}
	
	/**
	 * Method: onmouseup
	 *
	 *	This garantees that all panels and derivate classes have the
	 *	class set to <className> and <name>
	 *
	 */
	p.onmouseup = function (e)
	{
		var actual;
		if (!this.cssClasses || (actual = this.cssClasses[this.cssActualSelection]) == null) return;
		delete this.cssExtraClasses['onmousedown'];
		this.setCSSClass(this.cssActualSelection);
	}
